/***************************************************************************
                          kpogre.cpp  -  description
                             -------------------
    begin                : t en  6 16:08:39 CEST 2002
    copyright            : (C) 2002 by Dominik Sacher
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qinputdialog.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kedittoolbar.h>
#include <kkeydialog.h>

// application specific includes
#include "kpogre.h"
#include "kpogreview.h"
#include "kpogredoc.h"

#define ID_STATUS_MSG 1

KPoGreApp::KPoGreApp(QWidget* , const char* name):KMainWindow(0, name)
{
  config=kapp->config();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();
  initDocument();
  initView();
	
  readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  fileSave->setEnabled(false);
  fileSaveAs->setEnabled(false);
  filePrint->setEnabled(false);
  editCut->setEnabled(false);
  editCopy->setEnabled(false);
  editPaste->setEnabled(false);
}

KPoGreApp::~KPoGreApp()
{

}

void KPoGreApp::initActions()
{
  //-----------------------------------------------------------------
	// File menu

  fileConnectDb = new KAction(i18n("&Connect..."), "connect_established", 0, this, SLOT(slotConnectDb()), actionCollection(), "connect_established");
	fileDisconnectDb = new KAction(i18n("&Disconnect"), "connect_no", 0, this, SLOT(slotDisconnectDb()), actionCollection(), "connect_no");
  fileExportResult = new KAction(i18n("&Export Result..."), "fileexport", 0, this, SLOT(slotExportResult()), actionCollection(), "export_result");

	fileNewWindow = new KAction(i18n("New &Window"), 0, 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
  fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
  fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());

  fileNewWindow->setStatusText(i18n("Opens a new application window"));
  fileNew->setStatusText(i18n("Creates a new document"));
  fileOpen->setStatusText(i18n("Opens an existing document"));
  fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  fileSave->setStatusText(i18n("Saves the actual document"));
  fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
  fileClose->setStatusText(i18n("Closes the actual document"));
  filePrint ->setStatusText(i18n("Prints out the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));

  //-----------------------------------------------------------------
	// Edit menu
	editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());

  editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));

  //-----------------------------------------------------------------
	// View menu
	viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection(), "showtoolbar");
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection(), "showstatusbar");

  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

  viewEnableResultTab = new KAction(i18n("&Enable a new Result-Tab"), "news_subscribe", 0, this, SLOT(slotEnableResultTab()), actionCollection(), "enable_result_tab");
	viewDisableResultTab = new KAction(i18n("&Disable the last Result-Tab"), "news_unsubscribe", 0, this, SLOT(slotDisableResultTab()), actionCollection(), "disable_result_tab");
	viewClearInputArea = new KAction(i18n("&Clear Inputarea"), "editdelete", 0, this, SLOT(slotClearInputArea()), actionCollection(), "clear_input_area");

  //-----------------------------------------------------------------
	// Settings menu
	KStdAction::keyBindings(this, SLOT(slotConfigureKeys()), actionCollection(), "configurekeys");
  KStdAction::configureToolbars(this, SLOT(slotConfigToolbars()), actionCollection(), "configuretoolbars");
	//KStdAction::preferences(this, SLOT (slotConfigure()), actionCollection(), "configure");
	
  // use the absolute path to your kpogreui.rc file for testing purpose in createGUI();
  createGUI(); // "/home/lvanek/Projects/kpogre/kpogre/kpogreui.rc"
}


void KPoGreApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
}

void KPoGreApp::initDocument()
{
  doc = new KPoGreDoc(this);
  doc->newDocument();
	connect(doc, SIGNAL(sigModified()), this, SLOT(slotDocumentModified()));
}

void KPoGreApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new KPoGreView(this);
  doc->addView(view);
  setCentralWidget(view);	
  setCaption(doc->URL().fileName(),false);

}

void KPoGreApp::openDocumentFile(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));

  doc->openDocument( url);
  fileOpenRecent->addURL( url );
  slotStatusMsg(i18n("Ready."));
}


KPoGreDoc *KPoGreApp::getDocument() const
{
  return doc;
}

void KPoGreApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  fileOpenRecent->saveEntries(config,"Recent Files");
}


void KPoGreApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();


  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);
	
  // initialize the recent file list
  fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}

void KPoGreApp::saveProperties(KConfig *_cfg)
{
  if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
  {
    // saving to tempfile not necessary

  }
  else
  {
    KURL url=doc->URL();	
    _cfg->writeEntry("filename", url.url());
    _cfg->writeEntry("modified", doc->isModified());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);
    KURL _url(tempurl);
    doc->saveDocument(_url);
  }
}


void KPoGreApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  KURL url(filename);
  bool modified = _cfg->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL _url(tempname);
  	
    if(canRecover)
    {
      doc->openDocument(_url);
      doc->setModified();
      setCaption(_url.fileName(),true);
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(),false);
    }
  }
}		

bool KPoGreApp::queryClose()
{
  return doc->saveModified();
}

bool KPoGreApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KPoGreApp::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));
	
  KPoGreApp *new_window= new KPoGreApp();
  new_window->show();

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {	
    doc->newDocument();		
    setCaption(doc->URL().fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {	
    KURL url=KFileDialog::getOpenURL(QString::null,
        i18n("*.sql|SQL files"), this, i18n("Open File..."));
    if(!url.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(), false);
      fileOpenRecent->addURL( url );
    }
  }
  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified())
  {
     // here saving wasn't successful
  }
  else
  {
    doc->openDocument(url);
    setCaption(url.fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));
	
	if(doc->URL().fileName() == i18n("Untitled"))
  	slotFileSaveAs();
	else
		doc->saveDocument(doc->URL());
		
	setCaption(doc->URL().fileName(), doc->isModified());

  fileSave->setEnabled(false);

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileSaveAs()
{
  slotStatusMsg(i18n("Saving file with a new filename..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*.sql|SQL files"), this, i18n("Save as..."));
  if(!url.isEmpty())
  {
    doc->saveDocument(url);
    fileOpenRecent->addURL(url);
    setCaption(url.fileName(), doc->isModified());
  }

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
	
  close();

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));

  QPrinter printer;
  if (printer.setup(this))
  {
    view->print(&printer);
  }

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
	break;
    }
  }	
}

void KPoGreApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}


void KPoGreApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

/** Connect to Database Server */
void KPoGreApp::slotConnectDb()
{
	slotStatusMsg(i18n("Connect to Database ..."));
	view->connectSlot();
	slotStatusMsg(i18n("Ready."));
}

/** Disconnect Database */
void KPoGreApp::slotDisconnectDb()
{
	slotStatusMsg(i18n("Disconnect from Database"));
	view->disconnectSlot();
	slotStatusMsg(i18n("Ready."));
}
/** Enable new Result-Tab */
void KPoGreApp::slotEnableResultTab()
{
	slotStatusMsg(i18n("Enable a new Result-Tab"));
	view->addOutputTab();
	slotStatusMsg(i18n("Ready."));
}

/** Disable last Result-Tab */
void KPoGreApp::slotDisableResultTab()
{
	slotStatusMsg(i18n("Disable the last Result-Tab"));
	view->removeOutputTab();
	slotStatusMsg(i18n("Ready."));
}

/** Clear Input Area */
void KPoGreApp::slotClearInputArea()
{
	slotStatusMsg(i18n("Clear Inputarea"));
	view->clearSlot();
	slotStatusMsg(i18n("Ready."));
}
/** Dialog for kye binding configuration */
void KPoGreApp::slotConfigureKeys()
{
  KKeyDialog::configureKeys(actionCollection(), xmlFile());
}
/** Configute toolbars dialog */
void KPoGreApp::slotConfigToolbars()
{
  saveMainWindowSettings(KGlobal::config(), "KPogreMainWindow");
  KEditToolbar dlg(actionCollection());

	if (dlg.exec())
		{ createGUI();
		}
}
/** Save query result to file */
void KPoGreApp::slotExportResult()
{
	QString fileName = KFileDialog::getSaveFileName( QString(getenv("HOME") ), QString("*.tbl"), view, QString("Export query-result as ...") );
	if( fileName != QString::null ) {			// OK, User made or selected a filename
		QFile* file = new QFile( fileName );
  		if( file->exists() ){                   //is it already existing ?
  			QString error(fileName); 	error.append(" already exists!\n Do you wish to overwrite it?");
  	  		if (KMessageBox::questionYesNo(view, error, "Overwrite File?" )==KMessageBox::No) {
  	  			slotExportResult();					//user does not want to overwrite ! -> jump out and recall
  	  			return;
  	  		}
  	  	}
		
  	  	if ( ! file->open(IO_WriteOnly) ) {  // is he able to write to it ?
  	  		QString error("Error writing to ");
  	  		error.append(fileName);
  	  		error.append(" !");
  	  		KMessageBox::error( view, error, "Error"  );
  	  		return;
  	  	}
 		
  	  	else {   	  	                    //Yes, he is able to write !
  			QStringList list;
  			list << ";" << "|" << " ";
  			QString sep = QInputDialog::getItem( "Field Separator?", "Please choose a field separator!", list  );
  			QTextStream* out = new QTextStream( file );
     		QTable* outputArea = view->getCurrentlySelectedOutputTab();
			int cols = outputArea->numCols();
     		int rows = outputArea->numRows();
     		for( int currentRow=0 ; currentRow<rows ; currentRow++ ){
     			for (int currentCol=0 ; currentCol<cols ; currentCol++ ){
     				*out << (outputArea->text( currentRow, currentCol )).stripWhiteSpace() << sep;
     			}
				*out << "\n";
			}
			file->flush();
			file->close();
		}
	}
}
/** // Called, when document is modified. Enable Save button. */
void KPoGreApp::slotDocumentModified()
{
	fileSave->setEnabled(true);
	setCaption(doc->URL().fileName(), true);
}
