/***************************************************************************
                          kpogreview.cpp  -  description
                             -------------------
    begin                : Fre Sep  7 16:06:25 CEST 2001
    copyright            : (C) 2002 by Dominik Sacher
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>

// application specific includes
#include "kpogreview.h"
#include "kpogredoc.h"
#include "kpogre.h"

KPoGreView::KPoGreView(QWidget *parent, const char *name) : QWidget(parent, name)
{
  setBackgroundMode(PaletteBase);
  PGD = 0;
		
   m_pPixmapKey = new QPixmap(UserIcon("key.png"));
   m_pPixmapDatabase = new QPixmap(UserIcon("database.png"));
   m_pPixmapTable = new QPixmap(UserIcon("table.png"));

  areaLayout = new QVBoxLayout(this);

		//QFont* font = new QFont("Times",10);
		QColor* color = new QColor("white");
		
		//leftSplitter
		leftSplitter = new QSplitter(Qt::Horizontal, this);
		listView = new QListView(leftSplitter);
		listView->addColumn(i18n("<Server>"));
		listView->addColumn(i18n("<Fieldname>"));
		listView->addColumn(i18n("<Fieldtype>"));
		listView->setRootIsDecorated(true);
		listView->setAllColumnsShowFocus(true);
		listView->setShowSortIndicator(true);
		//listView->setFont(*font);
		connect(listView, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT  (treeDoubleClickedSlot(QListViewItem*)) );

		connect(listView, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)), this, SLOT  (treeRightButtonPressed(QListViewItem*, const QPoint&, int)) );


		//rightSplitter
  	rightSplitter = new QSplitter(Qt::Vertical, leftSplitter);
		outputTab = new QTabWidget(rightSplitter);
		
		//construct 10 tables on 10 tabs
		tables = new (QTable*)[10];
		tabIndex = 0;
		QTable* outputArea;
		for (int i=0 ; i<10 ; i++ ) {
			outputArea = new QTable( outputTab );
		  	outputArea->verticalHeader()->setResizeEnabled( FALSE );
			outputArea->setSelectionMode(QTable::NoSelection);
		
   			outputArea->horizontalHeader()->setBackgroundColor(*color);
			outputArea->verticalHeader()->setBackgroundColor(*color);
			outputArea->setBackgroundColor(*color);
			//outputArea->setFont(*font);
			tables[ i ] = outputArea;
			QString tabString = "[  ";
			tabString.append( QString::number(i+1) );
			tabString.append("  ]");
			outputTab->addTab(tables[ i ], tabString);		
		}
		
		//don t show the last 9 of them
		for( int i=9 ; i>0 ; --i ) {
			outputTab->setTabEnabled( tables[ i ], false );
		}
	
	  inputTab = new QTabWidget(rightSplitter);
	  inputArea = new QMultiLineEdit(inputTab, "inputArea");
		inputTab->addTab(inputArea, QIconSet(SmallIcon("openterm")), i18n("query"));
		inputArea->setReadOnly(true);
    connect(inputArea, SIGNAL(textChanged()), this, SLOT(slotInputAreaTextChanged()));

	 	historyArea = new QMultiLineEdit(inputTab, "historyArea");
		inputTab->addTab(historyArea, QIconSet(SmallIcon("history")), i18n("history"));
		historyArea->setReadOnly(true);
		
		rightSplitter->setResizeMode(outputTab, QSplitter::Stretch);
		rightSplitter->setResizeMode(inputTab, QSplitter::Stretch);

		leftSplitter->setResizeMode(listView, QSplitter::Stretch);
		leftSplitter->setResizeMode(rightSplitter, QSplitter::Stretch);

		//Buttons
		buttonBox = new QHBox(this);
		connectButton = new QPushButton("connect", buttonBox);
		QObject::connect( connectButton, SIGNAL(clicked()), this, SLOT(connectSlot()) );
		sendButton	= new QPushButton("&send", buttonBox);
		QObject::connect( sendButton, SIGNAL(clicked()), this, SLOT(sendSlot()) );
	  sendButton->setEnabled(false);
    clearButton	= new QPushButton("&clear", buttonBox);
		QObject::connect( clearButton, SIGNAL(clicked()), this, SLOT(clearSlot()) );
		disconnectButton 	= new QPushButton("disconnect", buttonBox);
		QObject::connect( disconnectButton, SIGNAL(clicked()), this, SLOT(disconnectSlot()) );
		disconnectButton->setEnabled(false);


		//globalLayout
		QValueList<int> qvl;
		qvl.append(372);
		qvl.append(93);
		rightSplitter->setSizes(qvl);

		qvl.clear();
		qvl.append(170);
		qvl.append(620);
		leftSplitter->setSizes(qvl);

		areaLayout->addWidget(leftSplitter, 5);
		areaLayout->addWidget(buttonBox, 0);

		connectDialog = new PGConnectDialog(this, i18n("Connect"), true);
}

KPoGreView::~KPoGreView()
{
  delete PGD;
	delete connectDialog;

  delete m_pPixmapKey;
  delete m_pPixmapDatabase;
  delete m_pPixmapTable;
}

KPoGreDoc *KPoGreView::getDocument() const
{
  KPoGreApp *theApp=(KPoGreApp *) parentWidget();

  return theApp->getDocument();
}

void KPoGreView::print(QPrinter *pPrinter)
{
  QPainter printpainter;
  printpainter.begin(pPrinter);
	
  // TODO: add your printing code here

  printpainter.end();
}

void KPoGreView::setSQLText(const QString sqlText)
{
	inputArea->clear();
	inputArea->setText( sqlText );
}

const QString KPoGreView::getSQLText() const
{
	return inputArea->text();
}

void KPoGreView::connectSlot()
{
			int result = connectDialog->exec();
			if( result==QDialog::Accepted && disconnectSlot() ) {
				QStringList* dataStringList = connectDialog->getData();
				if( (*dataStringList)[0] == "" ) {
					(*dataStringList)[0] = "localhost";					
				}
				
				if( (*dataStringList)[1] == ""){
					(*dataStringList)[1] = "5432";					
				}
				
				if( (*dataStringList)[2] == "") {
					(*dataStringList)[2] = "template1";
				}
				
				if( (*dataStringList)[3] == "" ) {								
					(*dataStringList)[3]	= "postgres";							
				}
			
				try{
					PGD = new PGDealer((*dataStringList)[0], (*dataStringList)[1], (*dataStringList)[2],
															(*dataStringList)[3], (*dataStringList)[4]);

					sendButton->setEnabled(true);
					disconnectButton->setEnabled(true);
					inputArea->setReadOnly(false);
					inputArea->setFocus();
					updateTreeSlot();
				}
				catch (PGDealerException pgde) {showException(pgde);}
	 		}

			else if(result==QDialog::Rejected){
				connectDialog->hide();
			}
		}



void KPoGreView::sendSlot (QString _query)
{
	 	bool needUpdateTree = false;
	 	const QString updateWords[] = {"drop", "alter", "delete", "create"};
      QTable* outputArea = (QTable*)outputTab->currentPage();
	 	
		for(int i=0 ; i<4 ; i++){
			if( _query.contains(updateWords[i],false)){ needUpdateTree = true; }
		}

		try{
				//QStringList** res = PGD->getResult(_query);
				QList<QStringList>* res =  PGD->getResult( _query );
				QStringList* qsl = new QStringList();
				int tempCount=-1;
				//outputArea->setNumRows( PGD->Tuples() );
				//outputArea->setNumCols ( PGD->Fields() );
				
				outputArea->setNumRows( res->count() );
				outputArea->setNumCols ( res->first()->count() );				

				QHeader* hHeader = outputArea->horizontalHeader();
				qsl = PGD->getCurrentFieldNames();

				//Header setzen
				for (QStringList::Iterator it = qsl->begin(); it != qsl->end(); ++it ) {
     				hHeader->setLabel(++tempCount,(*it).latin1());
				}

				//Zellen fllen
				for(unsigned int i=0 ; i<res->count() ; i++){
					qsl = res->at( i );
					tempCount=-1;
					for (QStringList::Iterator it = qsl->begin(); it != qsl->end(); ++it ) {
   					outputArea->setText(i,++tempCount,(*it).latin1());
 					}
				}
				
				for(int i=0 ; i<outputArea->numCols() ; i++){
					outputArea->adjustColumn(i);
				}
			
				//history fllen
				historyArea->append( inputArea->text() );
				
			}
			catch (PGDealerException pgde){	showException(pgde);	}
			if( needUpdateTree ) {
					updateTreeSlot();	
			}
}
	
	 	
	
void KPoGreView::sendSlot ( ) {
	sendSlot(inputArea->text());
}
	


void KPoGreView::clearSlot ( )
{
	inputArea->clear();
}
		

		
bool KPoGreView::disconnectSlot (QString _dbName)
{
	if (PGD) {																				//are we connected ?
		QString question = "Disconnect from ";
		question.append(_dbName);
		question.append(" ?");
		if( KMessageBox::questionYesNo(this, question, "question") == KMessageBox::Yes ) {	
			sendButton->setEnabled(false);							//yes, we are, but we like to disconnect !
			disconnectButton->setEnabled(false);
			for( int i=0 ; i<10 ; ++i ) {
				tables[ i ]->setNumRows(0);
				tables[ i ]->setNumCols(0);
			}
			for( int i=9 ; i>0 ; --i ) {
				outputTab->setTabEnabled( tables[ i ], false );
			}
			inputArea->clear();
			inputArea->setReadOnly(true);
			historyArea->clear();
			PGD = 0;
			updateTreeSlot();
			return true;
		}
		else {
			return false;                                              //yes, we are, but we don t want to disconnect !
		}
	}
	return true;                                                     //no, we are not connected, so just continue connecting
}
		


bool KPoGreView::disconnectSlot ( )
{
	if(PGD){                                                           //are we connected ?
		return disconnectSlot( *(PGD->getCurrentDBName()) );    //we are! do we like to disconnect ?
	}
	return true;                                                       //no, we are not connected, so just continue connecting
}



void KPoGreView::updateTreeSlot ( )
{
	listView->clear();
	if(PGD!=0){
   			QStringList* 			dbNames 		= PGD->getAllDBNames();
				QStringList* 			tableNames 	= PGD->getTableNames();
				QList<QStringList>* 		fieldList;
				QString* 				         currentDB		= PGD->getCurrentDBName();

				QStringList*			fields;
				QString					fieldName;
				QString					fieldType;
				QString       			notNull;

				QListViewItem*	parent;
				QListViewItem*	child;
				QListViewItem*	grandChild;


				listView->setColumnText(0,PGD->getHostname());
				//build parents (db Names)
				for (QStringList::Iterator dbIter = dbNames->begin(); dbIter != dbNames->end(); ++dbIter ) {
     				parent = new QListViewItem(listView, (*dbIter).latin1());
            parent->setPixmap(0, *m_pPixmapDatabase);


						//build childs (tablenames) of connected db
						if( (*dbIter).latin1() == (*currentDB) ){
							for (QStringList::Iterator tableIter = tableNames->begin() ;
																	tableIter != tableNames->end(); ++tableIter ) {
								child = new QListViewItem(parent, (*tableIter).latin1());
                child->setPixmap(0, *m_pPixmapTable);


								//build grandchilds (fieldTypes/fieldNames)
								fieldList 	= PGD->getFieldTypesForTable((*tableIter).latin1());
								for(unsigned int i=0 ; i<fieldList->count() ; i++){
									fields =  fieldList->at( i );
									QStringList::Iterator fieldIter = fields->begin();
									notNull 	= (*fieldIter).latin1();
									fieldName = (*(++fieldIter)).latin1();
									fieldType = (*(++fieldIter)).latin1();
									if(notNull=="t"){notNull="[PK]";}
									else{notNull="";}
									if(fieldType=="bpchar"){fieldType="varchar";}
									grandChild = new QListViewItem(child, notNull, fieldName, fieldType);
									//---------------
                           /*QString pwd ( getenv("PWD") );
									pwd.append( "/icons/pkey.xpm" );*/
									
									if( notNull=="[PK]" ){
										grandChild->setText(0,"");
										grandChild->setPixmap(0, *m_pPixmapKey);
									}

									//---------------
								}
							}
						}
				}
			}
}


	
void KPoGreView::treeDoubleClickedSlot (QListViewItem* clickedItem)
{
	QString* query;
	//fieldname doubleClicked
	if( clickedItem->parent() && PGD->getFieldNames(clickedItem->parent()->text(0))->contains(clickedItem->text(1)) ){
		query = new QString("select ");
		query->append(clickedItem->text(1));
		query->append(" from ");
		query->append(clickedItem->parent()->text(0));
		sendSlot(*query);
	}
}
	


void KPoGreView::treeRightButtonPressed (QListViewItem* pressedItem, const QPoint& point, int column) {
		if(column >= 0)
{
			//fieldname rightPressed
			QPopupMenu* popupMenu = new QPopupMenu();
			if( pressedItem->parent() && PGD->getFieldNames(pressedItem->parent()->text(0))->contains(pressedItem->text(1)) ){			
				popupMenu->insertItem("show all",0);
				popupMenu->insertSeparator();
				popupMenu->insertItem("AVG",1);
				popupMenu->insertItem("SUM",2);
				popupMenu->insertItem("COUNT",3);
				popupMenu->insertItem("MAX",4);
				popupMenu->insertItem("MIN",5);
				QObject::connect(popupMenu, SIGNAL( activated(int) ),
				                 this,      SLOT(   fieldNamePopupSlot(int) ) );
 				popupMenu->popup(point);
			}

			//tablename rightPressed
			else if (pressedItem->parent() && PGD->getTableNames()->contains(pressedItem->text(0))){
				popupMenu->insertItem("show all",0);
				popupMenu->insertSeparator();
				popupMenu->insertItem("delete all",1);
				popupMenu->insertItem("drop table",2);
				QObject::connect(popupMenu, SIGNAL( activated(int) ),
				                 this,      SLOT(   tableNamePopupSlot(int) ) );
 				popupMenu->popup(point);
			}


			//dbName rightpressed
			else if (PGD->getAllDBNames()->contains(pressedItem->text(0))){
				popupMenu->insertItem("connect to", 0);
				popupMenu->insertSeparator();
				popupMenu->insertItem("disconnect from", 1);
				
				if(  *PGD->getCurrentDBName() ==  pressedItem->text(0) ) {
   	      	popupMenu->setItemEnabled ( 0, false );
      		}
				else {
				 	popupMenu->setItemEnabled ( 1, false );
				}
      		
				QObject::connect(popupMenu, SIGNAL( activated(int) ),
				                 this,      SLOT(   dbNamePopupSlot(int) ) );
 				
 				popupMenu->popup(point);
			}

		}
}




void KPoGreView::dbNamePopupSlot (int index)
{
	if(index==0){ //connect to
		connectDialog->setDBName(listView->selectedItem()->text(0));
		connectSlot();
	}

	else if(index==1){//disconnect from
		disconnectSlot(listView->selectedItem()->text(0));
	}
}

void KPoGreView::tableNamePopupSlot (int index)
{
		QString* query = new QString();

		if(index==0){ 			//show all
			query->append("select * from ");
			query->append(listView->selectedItem()->text(0));
			sendSlot(*query);
		}

		else if(index==1){ 	//delete all
			QString q = "Delete all from ";
			q.append(listView->selectedItem()->text(0));
			q.append(" ?");
			if( (QMessageBox::warning(this,"WARNING", q, 1, 2))==1 ){
				query->append("delete from ");
				query->append(listView->selectedItem()->text(0));
				sendSlot(*query);
			}
		}


		else if(index==2){ 	//drop table
			QString q = "Drop table ";
			q.append(listView->selectedItem()->text(0));
			q.append(" ?");
			if( (QMessageBox::warning(this,"WARNING", q, 1, 2))==1 ){
				query->append("drop table ");
				query->append(listView->selectedItem()->text(0));
				sendSlot(*query);
			}
		}


}



void KPoGreView::fieldNamePopupSlot (int index)
{
		QString* query = new QString("select ");

		if(index==0){ //show all
			query->append(listView->selectedItem()->text(1));
		}

		else if(index==1){//avg
			query->append("avg(");
			query->append(listView->selectedItem()->text(1));
			query->append(")");
		}

		else if(index==2){//sum
			query->append("sum(");
			query->append(listView->selectedItem()->text(1));
			query->append(")");
		}

		else if(index==3){//count
			query->append("count(");
			query->append(listView->selectedItem()->text(1));
			query->append(")");
		}

		else if(index==4){//max
			query->append("max(");
			query->append(listView->selectedItem()->text(1));
			query->append(")");
		}

		else if(index==5){//min
			query->append("min(");
			query->append(listView->selectedItem()->text(1));
			query->append(")");
		}

		query->append(" from ");
		query->append(listView->selectedItem()->parent()->text(0));
		sendSlot(*query);
}
	


void KPoGreView::showException (PGDealerException pgde)
{
	if( !pgde.getSevere() ) {	//Statusbar
		 KPoGreApp *theApp=(KPoGreApp *) parentWidget();
		 theApp->slotStatusMsg( i18n(pgde.toString()) );
	}
	else {                           //Errordialog
     	KMessageBox::error(0, i18n(pgde.toString()) , "error");
	}
}	
	


void KPoGreView::addOutputTab()
{
 	if( tabIndex < 9 )
		{ outputTab->setTabEnabled (tables[++tabIndex], true );
   	}
}


void KPoGreView::removeOutputTab()
{
	if( tabIndex > 0 )
		{ outputTab->setTabEnabled (tables[tabIndex], false );
   		tabIndex--;
   	}	
}	


QTable* KPoGreView::getCurrentlySelectedOutputTab( )
{
	return (QTable*)outputTab->currentPage();	
}

/** Called, when user change text in SQL input area. Mark document as modyfied. */
void KPoGreView::slotInputAreaTextChanged()
{
	getDocument()->setModified();
}
