/***************************************************************************
                          kpogreview.h  -  description
                             -------------------
    begin                : Fre Sep  7 16:06:25 CEST 2001
    copyright            : (C) 2002 by Dominik Sacher
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPOGREVIEW_H
#define KPOGREVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qwidget.h>
#include <qlistview.h>
#include <qtextview.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qtable.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qsplitter.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qvaluelist.h>
#include <qdialog.h>
#include <qlineedit.h>
#include <qplatinumstyle.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qfont.h>
#include <qmessagebox.h>
#include <qtabwidget.h>
#include <qcombobox.h>

//includes files for kde
#include <kmessagebox.h>
#include <klocale.h>


//includes for application
#include "pgconnectdialog.h"
#include "pgdealer.h"

class KPoGreDoc;

/** The KPoGreView class provides the view widget for the KPoGreApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As KPoGreView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the KPoGreApp class to manipulate and display
 * the document structure provided by the KPoGreDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class KPoGreView : public QWidget
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
    KPoGreView(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~KPoGreView();

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a KPoGreApp instance as a parent
     * widget to get to the window document pointer by calling the KPoGreApp::getDocument() method.
     *
     * @see KPoGreApp#getDocument
     */
    KPoGreDoc *getDocument() const;

    /** contains the implementation for printing functionality */
    void print(QPrinter *pPrinter);
	
    /*
     * show the given exception in a messagebox or statusline
     */
   void showException (PGDealerException pgde);

  // Set the given sql-string from an sqlfile on inputArea
  void setSQLText(const QString sqlText);

	// Return SQL test from input area
	const QString getSQLText() const;


	/*
	*  return the currently selected Outputtab (e.g. for saving the content to a file
	*/
	QTable* getCurrentlySelectedOutputTab( );

 protected:
   QPixmap* m_pPixmapDatabase;
   QPixmap* m_pPixmapTable;
   QPixmap* m_pPixmapKey;


public slots:

   /*
	* disconnect from current connected database
	*/
	bool disconnectSlot();
	

	/**
	* enable a tab containing an qtable as a new outputarea to the qtabwidget
	*/
	void addOutputTab();
	
	
	/**
	* disable a tab containing an qtable as a new outputarea to the qtabwidget
	*/
	void removeOutputTab();	
	
	
	public slots:
	/*
	* connect after getting data from PGConnectdialog
	*/
	void connectSlot();
	
	/*
	* send QString (query) to server
	*/
	void sendSlot(QString);
	
	/*
	* send text from Inputarea to server
	*/
	void sendSlot();
	
	
	/*
	* clear inputarea
	*/
	void clearSlot();
	
	
	/*
	* disconnect from database with given name  ????????
	*/
	bool disconnectSlot(QString);
	
private slots:		
	/*
	* tree has changed and needs to update
	*/
	void updateTreeSlot();
	
	
	/*
	* perform "SELECT <doubleclickedtable> FROM <currentdb>"
	*/
	void treeDoubleClickedSlot(QListViewItem*);
	
	
	/*
	* show popup for "tablename" or "fieldname" in tree rightpressed
	*/
	void treeRightButtonPressed(QListViewItem*, const QPoint&, int);
	
	
	/*
	* show popup for databasename rightpressed for connect/disconnect to/from
	*/
	void dbNamePopupSlot(int);
	
	
	/*
	* decide what to do when user clicked in tablenamePopupMenu
	*/
	void tableNamePopupSlot(int);
	
	
	 /*
	* decide what to do when user clicked in fieldnamePopupMenu
	*/
	void fieldNamePopupSlot(int);
	
        	
  private:
 		PGDealer* 				PGD;
 		
 		QTabWidget*         inputTab;		
 		QMultiLineEdit* 		inputArea;
		QMultiLineEdit* 		historyArea;		
		
		QTabWidget*          	outputTab;
		QTable**      				tables;
		int									tabIndex;
		
		QPushButton*        	connectButton;
		QPushButton*        	sendButton;
		QPushButton*        	clearButton;
		QPushButton*        	disconnectButton;
   		
   	QSplitter*					rightSplitter;
		QSplitter*					leftSplitter;
		QVBoxLayout*      	areaLayout;
		QListView*          	listView;
		QHBox*						buttonBox;
		
		PGConnectDialog* 	connectDialog;
protected slots: // Protected slots
  /** Called, when user change text in SQL input area. Mark document as modyfied. */
  void slotInputAreaTextChanged();
};

#endif // KPOGREVIEW_H
