/***************************************************************************
                          		pgdealer.cpp  -  description
                             -------------------
    begin                : Fre Sep  7 16:06:25 CEST 2001
    copyright            : (C) 2001 by Dominik Sacher
    email                : Sacher@dvoid.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pgdealer.h"
#include <iostream.h>

/*
 * Konstruktor: erwartet diverse Attribute zum Connecten
 * todo: evtl. pgoptions pder hnliche ConnectOption einbauen
 */
PGDealer::PGDealer(QString _host="", QString _port="", QString _dbName="", QString _user="", QString _password="") :
			PgDatabase( (" host="     + _host +
		               " port="     + _port +
									 " dbname="   + _dbName +
							     " user="     + _user +
									 " password=" + _password).latin1() ){

										if( ConnectionBad() ){
											QString error = ErrorMessage();
											throw PGDealerException(error, true);
										}
										/*	for(int i=0 ; i<MAX_RESULT_TUPELS ; i++){
												result[i] = new QStringList();
											}
									   */
										_resultList = new QList<QStringList>();
											
									 	host=_host;
										port=_port;
										dbName=_dbName;
										user=_user;
										password=_password;
									 }

	
/*
 * Destruktor: muss evtl. noch mehr zerstrt werden ?
 */
PGDealer::~PGDealer() { _resultList->clear(); delete _resultList;}
	
	

/*
 * liefert QString der aktuell verbundenen Datenbank
 */
QString* PGDealer::getCurrentDBName() {
	return new QString(DBName());
}
	
	
	
	
/*
 * liefert QStringList ber alle vorhandenen Datenbanken, Zugriff ber Iterator,
 * sonst 0 (wenn keine Verbindung vorhanden)
 */
QStringList* PGDealer::getAllDBNames() {
	QStringList* resultStringList = new QStringList();
	if( !ConnectionBad() ){
		Exec("select * from pg_database");
		for(int i=0 ; i < Tuples() ; i++) {
			resultStringList->append(GetValue(i,0));		
		}
	}
	else{
		resultStringList = 0;
	}
	return resultStringList;
}
	


/*
 * liefert QStringList ber alle Tables der aktuellen DB
 */
QStringList* PGDealer::getTableNames() {
	QStringList* resultStringList = new QStringList();
	if( !ConnectionBad() ){
		Exec("SELECT relname FROM pg_class WHERE "
						"relkind = 'r' AND "
						"relname !~ '^pg_' AND "
						"relname !~ '^pga_'  ORDER BY relname");
		for(int i=0 ; i < Tuples() ; i++) {
			resultStringList->append(GetValue(i,0));		
		}
	}
	else{
		resultStringList = 0;
	}
	return resultStringList;
}


//select relname from pg_class where relname like 'l_lit_%'
//Damit bekommen wir Feldtypen und deren Datentyp
/*SELECT  a.attname, t.typname  FROM pg_class c, pg_attribute a, pg_type t
                  WHERE c.relname = 'l_lit' AND
                        a.attnum > 0 AND
                        a.attrelid = c.oid AND
                        a.atttypid = t.oid
                  ORDER BY attnum*/
QList<QStringList>* PGDealer::getFieldTypesForTable( QString tableName ){
	QList<QStringList>* resultStringList;
	QString* query = new QString("SELECT a.attnotnull, a.attname, t.typname  FROM "
																	"pg_class c, pg_attribute a, pg_type t "
       														"WHERE c.relname = '");
	query->append(tableName.latin1());
	query->append("' AND a.attnum > 0 AND a.attrelid = c.oid AND "
                        						"a.atttypid = t.oid "
                  									"ORDER BY attnum");
	try{
		resultStringList = getResult( *query );
	}
	catch (PGDealerException pgde) {throw pgde;}
	return resultStringList;
}


	
/*
 * liefert QStringList ber alle vorhandenen Feldnamen, Zugriff ber Iterator,
 * sonst 0 (wenn keine Verbindung vorhanden)
 */
QStringList* PGDealer::getCurrentFieldNames() {
	QStringList* resultStringList = new QStringList();
	if( !ConnectionBad() ){
		for(int i=0 ; i<Fields() ; i++){
			resultStringList->append( FieldName(i) );
		}
	}
	else{
		resultStringList = 0;
	}
	return resultStringList;
}
	
	
	
/*
 * liefert QStringList ber alle fieldnames des tables "tableName"
 */
QStringList* PGDealer::getFieldNames(QString tableName) {
	QStringList* result = new QStringList();
	QList<QStringList>* resultStringList;
	QString* query = new QString("SELECT a.attname FROM "
																	"pg_class c, pg_attribute a, pg_type t "
       														        "WHERE c.relname = '");
	query->append( tableName.latin1() );
	query->append("' AND a.attnum > 0 AND a.attrelid = c.oid AND "
                        						"a.atttypid = t.oid");

	try{
		resultStringList = getResult( *query );
	}
	catch (PGDealerException pgde) {throw pgde;}

	for(unsigned int i=0 ; i<resultStringList->count() ; i++){
		result->append(  resultStringList->at(i)->first().latin1()  );
	}
	return result;
}
	



/*
 * wichtigste Methode, liefert ein Array von Pointern auf die einzelnen
 * Zeilen (Tupel) zurck, die jeweils in einer QStringList stehen
 * speichert Ergebnis in der Membervariablen QStringList* result[]
 * todo: FEHLERBEHNANDLUNG OHNE ENDE
 */
/*
QStringList** PGDealer::getResult(QString query, bool old) {
	ExecStatusType status;
	if( !ConnectionBad() ){ 									// OK,Verbindung besteht
		status = Exec(query.latin1());
     if(status==PGRES_TUPLES_OK){				// alles bestens
			for (int zeilen=0 ; zeilen<Tuples() ; zeilen++) {
				result[zeilen] = new QStringList();
			}
			for (int zeilen=0 ; zeilen<Tuples() ; zeilen++) {
				for (int spalten = 0 ; spalten < Fields() ; spalten++) {
       			result[zeilen]->append(GetValue(zeilen, spalten));
	  			}
			}	
			return result;
		}			
					
		else if(status==PGRES_EMPTY_QUERY) {			// leere Query
			//QString error = ErrorMessage();
			QString error = "you sent an empty query !";
			throw PGDealerException(error, false);
		}
		
		else if(status==PGRES_COMMAND_OK) {			// leere Ergebnismenge
			//QString error = ErrorMessage();
		   QString error = "query did not return any data !";
			throw PGDealerException(error,false);
		}
		
		else if(status==PGRES_NONFATAL_ERROR) {	// wann wird der geworfen ???
			QString error = ErrorMessage();
    		cout << error << "nonfatal" << endl;
			cout << "\a \a \a \a";
			throw PGDealerException(error,false);
		}
			else if(status==PGRES_FATAL_ERROR) {		// Oo
			QString error = ErrorMessage();
			cout << "\a";
			throw PGDealerException(error,true);
		}
	}
	
	else{																			// keine Verbindung
		QString error = ErrorMessage();
		throw PGDealerException(error,true);
	}
}
*/



/*
* TESTME   XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
*/
QList<QStringList>* PGDealer::getResult( QString query ) {
	ExecStatusType status;
	if( !ConnectionBad() ){ 									// OK,Verbindung besteht
		status = Exec(query.latin1());
		_resultList->clear();
     if(status==PGRES_TUPLES_OK){				// alles bestens
			for (int zeilen=0 ; zeilen<Tuples() ; zeilen++) {
				_resultList->append( new QStringList() );
			}
			for (int zeilen=0 ; zeilen<Tuples() ; zeilen++) {
				for (int spalten = 0 ; spalten < Fields() ; spalten++) {
	  			   _resultList->at(zeilen)->append( GetValue( zeilen, spalten ) );
	  			}
			}	
			return _resultList;
		}			
					
		else if(status==PGRES_EMPTY_QUERY) {			// leere Query
			//QString error = ErrorMessage();
			QString error = "you sent an empty query !";
			throw PGDealerException(error, false);
		}
		
		else if(status==PGRES_COMMAND_OK) {			// leere Ergebnismenge
			//QString error = ErrorMessage();
		   QString error = "query did not return any data !";
			throw PGDealerException(error,false);
		}
		
		else if(status==PGRES_NONFATAL_ERROR) {	// wann wird der geworfen ???
			QString error = ErrorMessage();
    		cout << error << "nonfatal" << endl;
			cout << "\a \a \a \a";
			throw PGDealerException(error,false);
		}
			else if(status==PGRES_FATAL_ERROR) {		// Oo
			QString error = ErrorMessage();
			cout << "\a";
			throw PGDealerException(error,true);
		}
	}
	
	else{																			// keine Verbindung
		QString error = ErrorMessage();
		throw PGDealerException(error,true);
	}
}





/*
 * gibt das aktuelle Ergebnis aus der Membervariablen "result" aus
 */
/*
void PGDealer::showResult() {
	QStringList* qsl = new QStringList();
	for(int i=0 ; i<Tuples() ; i++){
		qsl = result[i];
		for (QStringList::Iterator it = qsl->begin(); it != qsl->end(); ++it ) {
     	cout << (*it).latin1() << " ";
  	}
		cout << endl;
	}
} */



